@@ -4,7 +4,7 @@
 
   Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 1997 - 2003 Pekka Riikonen
+  Copyright (C) 1997 - 2004 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: server_util.c,v 1.72.2.18 2003/12/16 20:40:47 pat Exp $ */
+/* $Id: server_util.c,v 1.72.2.23 2004/02/24 14:33:17 priikone Exp $ */
 
 #include "serverincludes.h"
 #include "server_internal.h"
@@ -119,6 +119,7 @@ silc_server_remove_clients_channels(SilcServer server,
       silc_hash_table_add(channels, channel, channel);
   }
   silc_hash_table_list_reset(&htl);
+  assert(!silc_hash_table_count(client->channels));
 }
 
 /* This function removes all client entries that are originated from
@@ -1024,6 +1025,7 @@ bool silc_server_channel_delete(SilcServer server,
 
     /* Totally delete the channel and all users on the channel. The
        users are deleted automatically in silc_idlist_del_channel. */
+    channel->disabled = TRUE;
     silc_schedule_task_del_by_context(server->schedule, channel->rekey);
     if (silc_idlist_del_channel(server->local_list, channel)) {
       server->stat.my_channels--;
@@ -1698,13 +1700,14 @@ silc_server_check_watcher_list_foreach(void *key, void *context,
 				  notify->client,
 				  notify->new_nick ? notify->new_nick :
 				  (const char *)notify->client->nickname,
-				  notify->notify);
+				  notify->notify,
+				  notify->client->data.public_key);
   }
 }
 
-/* This function checks whether the `client' nickname is being watched
-   by someone, and notifies the watcher of the notify change of notify
-   type indicated by `notify'. */
+/* This function checks whether the `client' nickname and/or 'client'
+   public key is being watched by someone, and notifies the watcher of the
+   notify change of notify type indicated by `notify'. */
 
 bool silc_server_check_watcher_list(SilcServer server,
 				    SilcClientEntry client,
@@ -1737,10 +1740,17 @@ bool silc_server_check_watcher_list(SilcServer server,
   n.new_nick = new_nick;
   n.notify = notify;
 
-  /* Send notify to all watchers */
+  /* Send notify to all watchers watching this nickname */
   silc_hash_table_find_foreach(server->watcher_list, hash,
 			       silc_server_check_watcher_list_foreach, &n);
 
+  /* Send notify to all watchers watching this public key */
+  if (client->data.public_key)
+    silc_hash_table_find_foreach(server->watcher_list_pk,
+				 client->data.public_key,
+			         silc_server_check_watcher_list_foreach,
+				 &n);
+
   return TRUE;
 }
 
@@ -1774,6 +1784,25 @@ bool silc_server_del_from_watcher_list(SilcServer server,
   }
   silc_hash_table_list_reset(&htl);
 
+  silc_hash_table_list(server->watcher_list_pk, &htl);
+  while (silc_hash_table_get(&htl, &key, (void *)&entry)) {
+    if (entry == client) {
+      silc_hash_table_del_by_context(server->watcher_list_pk, key, client);
+
+      if (client->id)
+	SILC_LOG_DEBUG(("Removing %s from WATCH list",
+			silc_id_render(client->id, SILC_ID_CLIENT)));
+
+      /* Now check whether there still exists entries with this key, if not
+	 then free the key to not leak memory. */
+      if (!silc_hash_table_find(server->watcher_list_pk, key, NULL, NULL))
+        silc_pkcs_public_key_free(key);
+
+      found = TRUE;
+    }
+  }
+  silc_hash_table_list_reset(&htl);
+
   return found;
 }
 
@@ -1925,8 +1954,8 @@ void silc_server_inviteban_process(SilcServer server, SilcHashTable list,
 	  if (len) {
 	    if (tmp[len - 1] == ',')
 	      tmp[len - 1] = '\0';
-	    silc_buffer_strformat(tmp2, tmp, SILC_STR_END);
-	    silc_buffer_strformat(tmp2, ",", SILC_STR_END);
+	    silc_buffer_strformat(tmp2, tmp, SILC_STRFMT_END);
+	    silc_buffer_strformat(tmp2, ",", SILC_STRFMT_END);
 	  }
 	} else {
 	  /* Announced list.  Check each entry in the list */
@@ -1963,8 +1992,8 @@ void silc_server_inviteban_process(SilcServer server, SilcHashTable list,
 
 	  /* Save the part that we didn't already have. */
 	  if (strlen(rtmp) > 1) {
-	    silc_buffer_strformat(tmp2, rtmp, SILC_STR_END);
-	    silc_buffer_strformat(tmp2, ",", SILC_STR_END);
+	    silc_buffer_strformat(tmp2, rtmp, SILC_STRFMT_END);
+	    silc_buffer_strformat(tmp2, ",", SILC_STRFMT_END);
 	  }
 	  silc_free(rtmp);
 	}
@@ -2245,7 +2274,7 @@ SilcStatus silc_server_set_channel_pk_list(SilcServer server,
   if (chpk && type == 0x03 && channel->channel_pubkeys &&
       server->server_type != SILC_ROUTER) {
     SilcBuffer sidp;
-    unsigned char mask[4];
+    unsigned char mask[4], ulimit[4];
 
     SILC_LOG_DEBUG(("Router enforces its list, remove old list"));
     silc_hash_table_free(channel->channel_pubkeys);
@@ -2254,8 +2283,10 @@ SilcStatus silc_server_set_channel_pk_list(SilcServer server,
     /* Send notify that removes the old list */
     sidp = silc_id_payload_encode(server->id, SILC_ID_SERVER);
     SILC_PUT32_MSB((channel->mode & (~SILC_CHANNEL_MODE_CHANNEL_AUTH)), mask);
+    if (channel->mode & SILC_CHANNEL_MODE_ULIMIT)
+      SILC_PUT32_MSB(channel->user_limit, ulimit);
     silc_server_send_notify_to_channel(server, NULL, channel, FALSE, TRUE,
-				       SILC_NOTIFY_TYPE_CMODE_CHANGE, 7,
+				       SILC_NOTIFY_TYPE_CMODE_CHANGE, 8,
 				       sidp->data, sidp->len,
 				       mask, 4,
 				       channel->cipher,
@@ -2267,7 +2298,13 @@ SilcStatus silc_server_set_channel_pk_list(SilcServer server,
 				       channel->passphrase,
 				       channel->passphrase ?
 				       strlen(channel->passphrase) : 0,
-				       NULL, 0, NULL, 0);
+				       NULL, 0, NULL, 0,
+				       (channel->mode &
+					SILC_CHANNEL_MODE_ULIMIT ?
+					ulimit : NULL),
+				       (channel->mode &
+					SILC_CHANNEL_MODE_ULIMIT ?
+					sizeof(ulimit) : 0));
     silc_buffer_free(sidp);
   }
 
