@@ -2,9 +2,9 @@
 
   serverid.h
 
-  Author: Pekka Riikonen <priikone@poseidon.pspt.fi>
+  Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 1997 - 2001 Pekka Riikonen
+  Copyright (C) 1997 - 2002 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,5 +31,8 @@ bool silc_id_create_client_id(SilcServer server,
 bool silc_id_create_channel_id(SilcServer server,
 			       SilcServerID *router_id, SilcRng rng,
 			       SilcChannelID **new_id);
+bool silc_id_is_valid_server_id(SilcServer server,
+				SilcServerID *server_id,
+				SilcSocketConnection remote);
 
 #endif
